/*-
 * Copyright (C)2008..2025 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ident "@(#) Copyright (C)2008..2025 @BABOLO http://www.babolo.ru/"
#ident "@(#) $Id: style_pgo0.c,v 1.120 2025/01/25 00:12:13 babolo Exp $"

/*     NFS    HVUDLR */

#define BLIN_COMPAT      4
#define Bpars_COMPAT     4
#define MULAR_COMPAT     0
#define MIFE_COMPAT      5
#define PGOBLIN_COMPAT   5
#define PGOBLIN_INTERNAL 1
#define PGOBLIN_MODULE   pgoblin_module

#include <sys/types.h>
#include <sys/event.h>
#include <sys/time.h>
#include <sysexits.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdio.h>
#include <err.h>
#include <babolo/BLINflag.h>
#include <babolo/parser.h>
#include <multilar.h>
#include <mife.h>
#include <pgoblin.h>

#define PGOBLIN_NTBSIZE  8                      /*       0-255 */
#define PGOBLIN_STYLE_ID "#pGoblin-pgo0"

#define H 0
#define U 1
#define D 2
#define V 3
#define L 4
#define R 5
#define N 6 /*   NULL,    L  R                        */
            /*        HVUDLR:                  */
#define F 7 /*   NULL,    L  R                         */
#define S 7 /*    ,    L  R (?) */
#define M 7
#define ENTITY_LETTERS 27

typedef struct pgo0_arb {             /*                          */
    char    *nm;
    ssize_t  vl;
} pgo0_arb;

typedef union pgo0_term {             /*   (!arbi)                              */
    u_int32_t index;                  /*                                                  */
    struct {
        u_int32_t numb   : 8          /*  entity                                                */
                , dflt   : 1          /*                                                    */
                , dummy1 : 7
                , type   : 8          /*  entity (H, U, D, V, L, R, N = 0 .. 6)                   */
                , dummy0 : 6
                , sign   : 1          /*  (+, -)                                                 */
                , arbi   : 1          /*   !!!MOST SIGNIFICANT BIT!!!                */
#   define D_ARBI 0x80000000          /*                                                         */
        ;
    };
} pgo0_term;

typedef struct pgo0_ent {
    u_int32_t  def;                   /* 0  NULL     mular pgoblin_realsty.str */
    struct {
        u_int32_t  f;                 /* 0  NULL     mular pgoblin_realsty.str */
        u_int32_t  b;                 /* 0  NULL     mular pgoblin_realsty.str */
    } spec[1 << PGOBLIN_NTBSIZE];
} pgo0_ent;

struct pgoblin_realsty {
    char              id[PGOBLIN_STRING_ID_LEN];  /*  "#pGoblin"             */
    u_int32_t         flags;
#   define            PGOBLIN_NTBSZMK (~(~0U << PGOBLIN_NTBSIZE)) /* 0x0000FFU */
#   define            FIRST                          0x000001U
#   define            FORWARD (FIRST   << 1)      /* 0x000002U */
#   define            LAST                           0x000004U
#   define            BAKWARD (LAST    << 1)      /* 0x000008U */
#   define            HSHIFT          12
#   define            VSHIFT          16
#   define            HFIRST  (FIRST   << HSHIFT) /* 0x001000U   Output +H0 when zero rows           */
#   define            HLAST   (LAST    << HSHIFT) /* 0x004000U   Output -H0 when zero rows           */
#   define            VFIRST  (FIRST   << VSHIFT) /* 0x010000U   Output +V0 when zero columns        */
#   define            VLAST   (LAST    << VSHIFT) /* 0x040000U   Output -V0 when zero columns        */
#   define            ALWAYS                         0x100000U
#   define            HALWAYS  ALWAYS             /* 0x100000U */
#   define            VALWAYS (ALWAYS  << 1)      /* 0x200000U */
#   define            EDGE    (HFIRST | HLAST | VFIRST | VLAST)
/*
 * pgoblin_realsty.flags
 * FIRST FORWARD
 *   0      0     @+0
 *   1      0   @+0   +0
 *   1      1   @+0  
 *
 * LAST  BAKWARD
 *   0      0     @-0
 *   1      0   @-0   -0
 *   1      1   @-0  
 */
#   define P_A (FIRST | LAST)
#if BLINAP > 4
    int32_t           dummy0;
#endif
    mular_descriptor *str;  /*  -                             */
    mular_descriptor *arb;  /*                                                */
    mular_descriptor *rid;  /*                                                     */
    pgo0_ent          a[M];
#if BLINAP > 4
    int32_t           dummy1;
#endif
};

extern pgoblin_styles PGOBLIN_MODULE;

static const u_char entity[ENTITY_LETTERS + 1] =
{ M, M, M, M,  D, M, F, M,  H, M, M, M,  L, M, N, M
, M, M, R, S,  M, U, V, M,  M, M, M, M
};

static const u_int32_t flagvalues[M] = {HALWAYS, HFIRST, HLAST, VALWAYS, VFIRST, VLAST};
static const char *clasnames[] = {"L0", "Ll", "Lb", "Ln", "Lz", "Ls", "Lu", "Le", "Ld", "Lo", "Lx"};

#define L0 0 /* \0           */
#define Ll 1 /*   */
#define Lb 2 /*        */
#define Ln 3 /*         */
#define Lz 4 /*    */
#define Ls 5 /*         */
#define Lu 6 /* +-           */
#define Le 7 /* =            */
#define Ld 8 /* #            */
#define Lo 9 /*     */
#define Lx 10

static const u_char class[256] =
{ L0, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lb, Ll, Ll,  Lo, Ll, Lo, Lo
, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
, Lb, Lo, Lo, Ld,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lu,  Lo, Lu, Lo, Lo
, Ln, Ln, Ln, Ln,  Ln, Ln, Ln, Ln,  Ln, Ln, Lo, Lo,  Lo, Le, Lo, Lo

, Lo, Ls, Ls, Ls,  Lz, Ls, Ls, Ls,  Lz, Ls, Ls, Ls,  Lz, Ls, Lz, Ls
, Ls, Ls, Lz, Ls,  Ls, Lz, Lz, Ls,  Ls, Ls, Ls, Lo,  Lo, Lo, Lo, Ls
, Lo, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls
, Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Lo,  Lo, Lo, Lo, Lo

, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
, Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
, Lo, Lo, Lo, Ls,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
, Lo, Lo, Lo, Ls,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo

, Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls
, Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls
, Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls
, Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls,  Ls, Ls, Ls, Ls
};

static const char statenames[][3] =
{ "bg", "bf", "bs", "nl",  "nq", "tb", "tp", "sz"
, "ss", "sn", "sc", "cl",  "cq", "mc", "ml", "tq"
, "id", "in", "ii", "xx"
};

enum states
{ bg /*                                                    */
, bf /*                                                      */
, bs /*                                      */
, nl /*                                           */
, nq /*     ?                       */
, tb /*                             */
, tp /*                                           */
, sz /*   +-                                               */
, ss /*                                        */
, sn /*                                     */
, sc /*       ()          */
, cl /*                                 */
, cq /* ?                                */
, mc /*                                                 */
, ml /*                                           */
, tq /*   ?                                 */
, id /*                                     */
, in /*                                        */
, ii /*                                */
, xx
};

static const char pn[][3] =
{ "- ", "- ", "- ", "- ", "- ", "- ", "- ", "- "
, "Cr", "Cj", "Cv", "Cm", "Ct", "Ci", "Cn", "Cb"
, "Ce", "Cc", "Cf", "Ck", "Cs", "Cd", "- ", "Cx"
};
#define Cr  0x800000  /*                                                            */
#define Cj  0x400000  /*                                                */
#define Cv  0x200000  /*                                                  */
#define Cm  0x100000  /*                                                       */
#define Ct  0x080000  /* s_num = 0   mular pgoblin_realsty.rid            */
#define Ci  0x040000  /*                                                                */
#define Cn  0x020000  /*                                                                    */
#define Cb  0x010000  /*                                                       */
#define Ce  0x008000  /*                                                        */
#define Cc  0x004000  /*                                                               */
#define Cp  (Cb | Cc) /*                                                */
#define Cf  0x002000  /*                                                                    */
#define Ck  0x001000  /*                                                                */
#define Cs  0x000800  /*                                                           */
#define Cd  0x000400  /*  , s_num = 0,  pgoblin_realsty.str      */
#define Cx  0x000100  /*                                                                    */
#define C_state 0xFF  /*                                                           */

static u_int32_t automa[xx][Lx] =
/*       \0        \n        \b      0..9         T         a        +-      =      #     .. */
/*       L0        Ll        Lb        Ln        Lz        Ls        Lu     Le     Ld     Lo */
{{       xx,       nl,       bg,       bs,    Cf|bf,       bs,       bs,    bs,    bs,    bs }/*bg*/
,{       xx,       nl,       bs,       bs,    Cf|bf,       bs,       bs,    bs,    bs,    bs }/*bf*/
,{       xx,       nl,       bs,       bs,       bs,       bs,       bs,    bs,    bs,    bs }/*bs*/
,{       xx,       nl,       bg,       bg,       bg,       bg,       bg,    bg,    nq,    bg }/*nl*/
,{       xx,       nl,       tb,       xx,       xx,       xx,    Cd|sz,    id,    bg, Cx|xx }/*nq*/
,{    Ck|xx,    Ck|nl,       tb,    Cx|xx, Cs|Cd|tp, Cs|Cd|tp,    Cd|sz, Ck|id, Ck|xx, Ck|xx }/*tb*/
,{       xx, Cm|   cl, Cm|   sc,       tp,       tp,       tp,    Cx|xx, Cx|xx, Cx|xx, Cx|xx }/*tp*/
,{    Cx|xx,    Cx|xx,    Cx|xx,    Cx|xx,    Ci|ss,    Cx|xx,    Cx|xx, Cx|xx, Cx|xx, Cx|xx }/*sz*/
,{ Cr|Ck|xx, Cr|   cl, Cr|   sc, Cn|   sn,    Cx|xx,    Cx|xx,    Cx|xx, Cx|xx, Cx|xx, Cx|xx }/*ss*/
,{ Cr|Ck|xx, Cr|   cl, Cr|   sc, Cn|   sn,    Cx|xx,    Cx|xx,    Cx|xx, Cx|xx, Cx|xx, Cx|xx }/*sn*/
,{ Cr|Ck|xx,       cl,       sc,       sc,       sc,       sc,       sc,    sc,    sc,    sc }/*sc*/
,{    Ck|xx, Cp|Ce|ml, Cp|   mc, Cp|   mc, Cp|   mc, Cp|   mc, Cp|   mc, Cp|mc,    cq, Cp|mc }/*cl*/
,{    Ck|xx, Ck|   nl,       tb, Ck|   xx, Ck|   xx, Ck|   xx, Ck|   xx, Ck|xx, Cp|mc, Ck|xx }/*cq*/
,{ Ce|Ck|xx, Ce|Cc|ml,    Cc|mc,    Cc|mc,    Cc|mc,    Cc|mc,    Cc|mc, Cc|mc, Cc|mc, Cc|mc }/*mc*/
,{    Ck|xx, Ce|Cc|ml,    Cc|mc,    Cc|mc,    Cc|mc,    Cc|mc,    Cc|mc, Cc|mc,    tq, Cc|mc }/*ml*/
,{    Ck|xx, Ck|   nl, Ck|   tb, Ck|   xx, Ck|   xx, Ck|   xx, Ck|Cd|sz, Ck|xx, Cc|mc, Ck|xx }/*tq*/
,{       xx,       nl,       id, Ct|Cn|in, Cs|Ct|ii, Cs|Ct|ii,    Cx|xx, Cx|xx, Cx|xx, Cx|xx }/*id*/
,{    Cj|xx,    Cj|nl,    Cj|id,    Cn|in,    Cx|xx,    Cx|xx,    Cx|xx, Cx|xx, Cx|xx, Cx|xx }/*in*/
,{    Cv|xx,    Cv|nl,    Cv|id,       ii,       ii,       ii,    Cx|xx, Cx|xx, Cx|xx, Cx|xx }/*ii*/
};

static size_t msz[] = {1024, 512, 512};
static size_t asz[] = {512, 512, 512};

static void dump(pgoblin_rst *rst, FILE *ouf);

static int
/**********************************************************************
 **                                                                  **/
getind(mular_descriptor *md, const char *stringname, int len) {     /**
 **                                                                  **
 **********************************************************************
 **    pgo0_arb   stringname           **
 **********************************************************************/
    char     *sname;
    pgo0_arb *arbp;
    u_int     i;

    if  (len < 0) {
        sname = strdup(stringname);
    } else {
        sname = strndup(stringname, (size_t)len);
    }
    for (i = 0; i < MULAR_NEXT(md); i++) {
        arbp = mular_getix(md, i);
        if  (!strcmp(arbp->nm, sname)) return((int)i);
    }
    if  (md->flags & MULAR_FIXD) return(-1);
    i = (u_int)MULAR_NEXT(md);
    arbp = mular_add(md);
    if  (!arbp) {
        /* XXXX */
    }
    arbp->nm = sname;
    return((int)i);
}

static int
/**********************************************************************
 **                                                                  **/
parser(pgoblin_rst *rst, char *srcstyle) {                          /**
 **                                                                  **
 **********************************************************************/
#   define blin_internal_flags (rst->flags & BLIN_MASK)
    u_int32_t         control;         /*                  */
    int               lfcount;         /*                                                */
    size_t            offset;          /*                        */
    char             *literal = NULL;  /*               */
    enum states       state;           /*                           */
    char             *l_end   = NULL;  /*                                   */
    char             *l_beg   = NULL;  /*                                    */
    u_int32_t         s_num   = 0;     /*                                                  */
    size_t            o_beg   = 0;     /*                                        */
    u_char            s_lit   = M;     /*  entity                                                */
    int               type    = 0;     /*  +(>0)  -(<0)    (&1)  entity     */
    mular_descriptor *ids     = NULL;  /*            */
    int               ex      = EX_OK; /*                                                 */

    if  (!srcstyle) {
        ifBLIN_QX0("No pgoblin style");
        ERROUT(EX_DATAERR, EINVAL);
    }
    if  (!rst->style) {
        if  (!(rst->style = calloc(1, sizeof(pgoblin_realsty)))) {
            ifBLIN_QX0("No memory #1");
            ERROUT(EX_OSERR, ENOMEM);
        }
        strncpy(rst->style->id, PGOBLIN_STYLE_ID, strlen(PGOBLIN_STYLE_ID));
    } else {
        if  (!!strncmp(rst->style->id, PGOBLIN_STYLE_ID, strlen(PGOBLIN_STYLE_ID))) {
            ifBLIN_QX0("Illegal style %s", rst->style->id);
            ERROUT(EX_DATAERR, EINVAL);
        }
        if  ((rst->style->flags & PGOBLIN_NTBSZMK) != PGOBLIN_NTBSIZE) {
            ifBLIN_QX0("Illegal style size %d", rst->style->flags & PGOBLIN_NTBSZMK);
            ERROUT(EX_DATAERR, EINVAL);
    }   }
    if  (  !rst->style->str
        && !(rst->style->str = mular_create(MULAR_STRI | MULAR_CHAR, 3, sizeof(char*), msz))
        ) {
        ifBLIN_QW0("mular_create #1");
        ERROUT(EX_OSERR, ENOMEM);
    }
    rst->style->flags = PGOBLIN_NTBSIZE;
    if  (  !rst->style->arb
        && !(rst->style->arb = mular_create( MULAR_STRI
                                           | MULAR_ZERO
                                           | MULAR_HEXU
                                           | MULAR_SWP1
                                           | MULAR_SWP2
                                           , 3
                                           , sizeof(pgo0_arb)
                                           , asz
        )   )                              ) {
        ifBLIN_QW0("mular_create #2");
        ERROUT(EX_OSERR, ENOMEM);
    }
    rst->style->arb->flags &= ~MULAR_FIXD;
    blin_stateheader(BLIN_5STO24G, pn);
    lfcount = 1;
    for (state = bg, offset = 0, control = 0; state < xx; offset++, state = control & C_state) {
        if  (srcstyle[offset] == '\n') lfcount++;
        control = automa[state][class[srcstyle[offset] & 0xFF]];
        blin_statebody( BLIN_5STO24G
                      , pn
                      , statenames
                      , clasnames[class[srcstyle[offset] & 0xFF]]
                      , srcstyle
                      , offset
                      , control
                      , state
                      , 0xFFFFFFFF
                      , (u_char)s_num
                      );
        if  (control & Cr) {
            u_int32_t entnum;         /*  entity                                                */
            pgo0_term term;

            if  (!(s_num < (1 << PGOBLIN_NTBSIZE))) {
                ifBLIN_QX0("Big number pgo0 line %d", lfcount);
                ERROUT(EX_DATAERR, EINVAL);
            }
            entnum = entity[s_lit - '@'];
            ifBLIN_QO1 {
                if  (type & 1) {
                    if  (rst->style->a[entnum].def)
                        ifBLIN_QX1( "Repeat %c%c in line %d\n"
                                  , (type > 0) ? '+' : '-', s_lit
                                  , lfcount
                                  );
                } else if ( (type < 0)
                          ? rst->style->a[entnum].spec[s_num].b
                          : rst->style->a[entnum].spec[s_num].f
                          ) {
                    ifBLIN_QX1( "Repeat %c%c%d in line %d\n"
                              , (type > 0) ? '+' : '-', s_lit, (int)s_num
                              , lfcount
                              );
            }   }
            ifBLIN_QX5("Cr add [%"BLIN_D"]", MULAR_NEXT(ids));
            term.index = 0;
            term.arbi = 1;
            term.sign = (type < 0);
            term.type = entnum;
            term.dflt = type & 1;
            term.numb = s_num;
            *(pgo0_term*)mular_add(ids) = term;
            ifBLIN_QX5("%08X", *(u_int32_t*)mular_getix(ids, MULAR_NEXT(ids) - 1));
            l_beg = NULL;
            l_end = NULL;
        }
        if  (control & Cj) {
            if  (s_num & D_ARBI) {
                ifBLIN_QX0("Overflow line %d", lfcount);
                ERROUT(EX_DATAERR, EINVAL);
            }
            *(u_int32_t*)mular_add(rst->style->rid) = s_num;
        }
        if  (control & Cv) {
            u_int32_t idx;

            srcstyle[offset] = '\0';
            idx = (u_int32_t)MULAR_NEXT(rst->style->str);
            if  (idx & D_ARBI) {
                ifBLIN_QX0("Overflow line %d", lfcount);
                ERROUT(EX_DATAERR, EINVAL);
            }
            *(char**)(mular_add(rst->style->str)) = &srcstyle[o_beg];
            *(u_int32_t*)mular_add(rst->style->rid) = idx | D_ARBI;
        }
        if  (control & Cm) {
            u_int32_t arbidx;

            srcstyle[offset] = '\0';
            arbidx = (u_int32_t)getind(rst->style->arb, &srcstyle[o_beg], -1);
            if  (arbidx & D_ARBI) {
                ifBLIN_QX0("Overflow line %d", lfcount);
                ERROUT(EX_DATAERR, EINVAL);
            }
            if  (((pgo0_arb*)mular_getix(rst->style->arb, arbidx))->vl) {
                ifBLIN_QX1("Repeat %s in line %d", &srcstyle[o_beg], lfcount);
            }
            ifBLIN_QX5("Cm add [%"BLIN_D"]", MULAR_NEXT(ids));
            *(u_int32_t*)mular_add(ids) = arbidx;
            ifBLIN_QX5("%08X", *(u_int32_t*)mular_getix(ids, MULAR_NEXT(ids) - 1));
            l_beg = NULL;
            l_end = NULL;
        }
        if  (control & Ct) {
            s_num = 0;
            if  (!rst->style->rid) {
                rst->style->rid = mular_create(MULAR_STRI, 3, sizeof(u_int32_t), msz);
        }   }
        if  (control & Ci) s_lit = (u_char)srcstyle[offset];
        if  (control & Cn) {
            s_num = s_num * 10 + (u_int)(srcstyle[offset] - '0');
            if  (s_num >= 0x10000000) {
                ifBLIN_QX0("Long number %u @ pgo0 line %d", s_num, lfcount);
                ERROUT(EX_DATAERR, EINVAL);
            }
            if  (type & 0xFF) type <<= 1;
        }
        if  (control & Cb && !l_beg) l_beg = literal = &srcstyle[offset];
        if  (control & Ce && l_beg) l_end = literal;
        if  (control & Cc) {
            if  (literal != &srcstyle[offset]) *literal = srcstyle[offset];
            literal++;
        }
        if  (control & Cf) {
            u_int32_t nwfl = flagvalues[entity[srcstyle[offset] - '@']];
            if  (rst->style->flags & nwfl & EDGE) nwfl <<= 1;
            rst->style->flags |= nwfl;
        }
        if  ((control & Ck) && !!ids) {
            u_int32_t  n = 0;

            if  (l_beg && l_end) {
                *l_end = '\0';
                n = (u_int32_t)MULAR_NEXT(rst->style->str);
                *(char**)(mular_add(rst->style->str)) = l_beg;
            }
            for (u_int k = 0; k < MULAR_NEXT(ids); k++) {
                int        entname = 0;/*  entity                                                 */
                pgo0_term *term;

                term = mular_getix(ids, k);
                ifBLIN_QO5 {
                    for (entname = 0; entname < ENTITY_LETTERS; entname++) {
                        if  (term->type == entity[entname]) break;
                    }
                    entname += '@';
                }
                if  (!(term->arbi)) {
                    ifBLIN_QX5("Ck [%u]%08X index %d", k, term->index, term->index);
                    ((pgo0_arb*)mular_getix( rst->style->arb
                                           , term->index
                    )                      )->vl = l_end ? (ssize_t)~n : 0;
                } else if (term->dflt) {
                    ifBLIN_QX5("Ck [%u]%08X D_DFLT D_TYPE %c", k, term->index, entname);
                    rst->style->a[term->type].def = l_end ? ~n : 0;
                } else {
                    ifBLIN_QX5( "Ck [%u]%08X sign=%c type=%c numb=%d"
                              , k, term->index
                              , (term->sign) ? '-' : '+'
                              , entname
                              , term->numb
                              );
                    if  (term->sign) {
                        rst->style->a[term->type].spec[term->numb].b = l_end ? ~n : 0;
                    } else {
                        rst->style->a[term->type].spec[term->numb].f = l_end ? ~n : 0;
            }   }   }
            mular_destroy(ids);
            ids = NULL;
        }
        if  (control & Cs) o_beg = offset;
        if  (control & Cd) {
            s_num = 0;
            if  (srcstyle[offset] == '-') type = -1; else type = 1;
            if  (!ids) ids = mular_create(MULAR_STRI, 3, sizeof(u_int32_t), msz);
        }
        if  (control & Cx) {
            ifBLIN_QX0("Erroneous line %d", lfcount);
            ERROUT(EX_DATAERR, EINVAL);
    }   }
out:
    if  (rst->style->arb) rst->style->arb->flags |= MULAR_FIXD;
    if  (ids) mular_destroy(ids);
    return(ex);
#   undef blin_internal_flags
}

static void
/**********************************************************************
 **                                                                  **/
dump(pgoblin_rst *rst, FILE *ouf) {                                 /**
 **                                                                  **
 **********************************************************************/
#   define blin_internal_flags (rst->flags & BLIN_MASK)
    int              ex = EX_OK;
    pgoblin_realsty *st;
    pgo0_arb        *e;
    u_char         **c;
    u_int            u;
    int              i;
    int              j;
    int              k;
   /******************************************************/
    u_int       tmpidx;  /* Due to the error in cc -v    *
    *** Using built-in specs.                          ***
    *** Configured with: FreeBSD/amd64 system compiler ***
    *** Thread model: posix                            ***
    *** gcc version 3.4.6 [FreeBSD] 20060305           ***
    * I can't use direct                                 *
    *** mular_getix(st->str, ~st->a[i].def)            ***
    ******************************************************/

    if  (!(st = rst->style)) {
        fprintf(ouf, "Dump pgo0: no style\n");
        ERROUT(EX_DATAERR, EINVAL);
    }
    if  ((st->flags & PGOBLIN_NTBSZMK) != PGOBLIN_NTBSIZE) {
        fprintf(ouf, "Illegal pgoblin pgo0 style size %d\n", st->flags & PGOBLIN_NTBSZMK);
        ERROUT(EX_DATAERR, EINVAL);
    }
    fprintf(ouf, "+######################################");
    for (i = 0; i < PGOBLIN_STRING_ID_LEN && st->id[i]; i++) fprintf(ouf, "%c", st->id[i]);
    fprintf(ouf, "\n flags=%08X %08X\n", rst->flags, st->flags);
    ifBLIN_QO2 {
        fprintf(ouf, "------->str[%d]\n", (int)MULAR_NEXT(st->str));
        ifBLIN_QO3 mular_dump(st->str, ouf);
        for (u = 0; u < MULAR_NEXT(st->str); u++) {
            c = mular_getix(st->str, u);
            if  (!c) fprintf(ouf, "[%d]\n", u);
            else if (!*c) fprintf(ouf, "[%u]%" BLIN_X "\n", u, BLIN_I(c));
            else fprintf(ouf, "[%u]=%s~\n", u, *c);
        }
        fprintf(ouf, "------->arb[%d]\n", (int)MULAR_NEXT(st->arb));
        ifBLIN_QO3 mular_dump(st->arb, ouf);
        for (u = 0; u < MULAR_NEXT(st->arb); u++) {
            e = mular_getix(st->arb, u);
            if  (!e) fprintf(ouf, "[%u]\n", u);
            else if (e->nm) fprintf(ouf, "%u[%s]%d\n", u, e->nm, (int)~e->vl);
            else fprintf(ouf, "%u[]%d\n", u, (int)~e->vl);
    }   }
    for (u = 0; u < MULAR_NEXT(st->arb); u++) {
        e = mular_getix(st->arb, u);
        tmpidx = (u_int)(~e->vl);
        fprintf(ouf, "# %s\n%s\n", e->nm, *(u_char**)mular_getix(st->str, tmpidx));
    }
    for (i = 0; i < M; i++) {
        for (j = 0; j < ENTITY_LETTERS; j++) if  (i == entity[j]) break;
        j += '@';
        if  (~(tmpidx = ~st->a[i].def)) {
            fprintf(ouf, "# +%c\n%s\n", j, *(u_char**)mular_getix(st->str, tmpidx));
        }
        for (k = 0; k < (1 << PGOBLIN_NTBSIZE); k++) {
            if  (~(tmpidx = ~st->a[i].spec[k].f)) {
                fprintf(ouf, "# +%c%d\n%s\n", j, k, *(u_char**)mular_getix(st->str, tmpidx));
            }
            if  (~(tmpidx = ~st->a[i].spec[k].b)) {
                fprintf(ouf, "# -%c%d\n%s\n", j, k, *(u_char**)mular_getix(st->str, tmpidx));
    }   }   }
    ifBLIN_QO3 {
        for (i = 0; i < M; i++) {
            for (j = 0; j < ENTITY_LETTERS; j++) if  (i == entity[j]) fprintf(ouf, "%c", j + '@');
            fprintf(ouf, "%c", st->a[i].def ? '|' : '.');
            for (j = 0; j < 76; j++)
                fprintf( ouf
                       ,  "%c"
                       , st->a[i].spec[j].f
                       ? (st->a[i].spec[j].b ? '*' : '+')
                       : (st->a[i].spec[j].b ? '-' : '.')
                       );
            fprintf(ouf, "\n");
    }   }
    fprintf(ouf, "-######################################\n");
    fflush(ouf);
out:
    return;
#   undef blin_internal_flags
}

static int
/**********************************************************************
 **                                                                  **/
clear(pgoblin_rst *rst) {                                           /**
 **                                                                  **
 **********************************************************************/
#   define blin_internal_flags (rst->flags & BLIN_MASK)
    pgo0_arb        *arbp;
    int              ex = EX_OK;
    pgoblin_realsty *st;

    if  (!!rst) {
        if  ((st = rst->style)) {
            if  (!!strncmp(st->id, PGOBLIN_STYLE_ID, strlen(PGOBLIN_STYLE_ID))) {
                ifBLIN_QX0("Illegal style %s", st->id);
                ERROUT(EX_DATAERR, EINVAL);
            }
            if  ((st->flags & PGOBLIN_NTBSZMK) != PGOBLIN_NTBSIZE) {
                ifBLIN_QX0("Illegal style size %d", st->flags & PGOBLIN_NTBSZMK);
                ERROUT(EX_DATAERR, EINVAL);
            }
            if  (st->str) mular_destroy(st->str);
            if  (st->arb) {
                for (size_t i = 0; i < MULAR_NEXT(st->arb); i++) {
                    arbp = mular_getix(st->arb, i);
                    free(arbp->nm);
                }
                mular_destroy(st->arb);
            }
            free(st);
            st = NULL;
            MARK_R_STYLE_GO(rst);;
            ;;  rst->flags &= ~PGOBLIN_STYLE_TYPE;
            MARK_R_STYLE_WENT(rst);;
    }   }
out:
    return(ex);
#   undef blin_internal_flags
}

static char *
/**********************************************************************
 **                                                                  **/
get(pgoblin_exenv *exenv, pgoblin_nr *rn) {                         /**
 **                                                                  **
 **********************************************************************/
#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
    char            *found = NULL;
    pgo0_arb        *arbp;
    pgoblin_rst     *rst;
    pgoblin_rio     *rct;
    int              ex = EX_OK;
    pgoblin_realsty *st;
    int              i;
   /******************************************************/
    u_int       tmpidx;  /* Due to the error in cc -v    *
    *** Using built-in specs.                          ***
    *** Configured with: FreeBSD/amd64 system compiler ***
    *** Thread model: posix                            ***
    *** gcc version 3.4.6 [FreeBSD] 20060305           ***
    * I can't use direct                                 *
    *** mular_getix(st->str, ~arbp->vl)                ***
    ******************************************************/

    GET_STY(rst, exenv->options, rn[PGO_CSTY]);
    st = rst->style;
    if  (!!strncmp(st->id, PGOBLIN_STYLE_ID, strlen(PGOBLIN_STYLE_ID))) {
        ifBLIN_QX0("Illegal style %s", st->id);
        ERROUT(EX_DATAERR, EINVAL);
    }
    if  ((st->flags & PGOBLIN_NTBSZMK) != PGOBLIN_NTBSIZE) {
        ifBLIN_QX0("Illegal style size %d", st->flags & PGOBLIN_NTBSZMK);
        ERROUT(EX_DATAERR, EINVAL);
    }
    GET_RIO(rct, exenv->options, rn[PGO_CCTL]);
    if  (!!rct->binparm) {
        i = getind(st->arb, rct->text, (int)rct->length);
    } else {
        i = getind(st->arb, rct->text, -1);
    }
    if  (i >= 0) {
        arbp = mular_getix(st->arb, (size_t)i);
        if  (arbp->vl) {
            tmpidx = (u_int)(~(arbp->vl));
            errno = 0;
            if  (!(found = strdup(*(char**)mular_getix(st->str, tmpidx)))) errno = ENOMEM;
    }   }
out:
    return(found);
#   undef blin_internal_flags
}

static ssize_t
/**********************************************************************
 **                                                                  **/
sout(pgoblin_rio *rou, char *c, ssize_t l) {                        /**
 **                                                                  **
 **********************************************************************/
    ssize_t ex = 0;

    if  (c) ex = pgoblin_io_write(rou, c, !!l ? (size_t)l : strlen(c));
    return(ex);
}

static int *
/**********************************************************************
 **                                                                  **/
makerenum(pgoblin_exenv *exenv, pgoblin_rst *rst, pgoblin_nr nin) { /**
 **                                                                  **
 **********************************************************************/
#   define blin_internal_flags ((rst ? rst->flags : pgoblin_default) & BLIN_MASK)
    int             *res = NULL;
    pgoblin_rio     *rin;
    pgoblin_realsty *st  = rst->style;

    GET_RIO(rin, exenv->options, nin);
    if  (!!st->rid) {
        res = malloc(MULAR_NEXT(st->rid) * sizeof(int));
        memset(res, -1, MULAR_NEXT(st->rid) * sizeof(int));
        for (u_int i = 0; i < MULAR_NEXT(st->rid); ++i) {
            u_int32_t *reix = mular_getix(st->rid, i);

            if  (!reix) {
                ifBLIN_QX1("Field %u is not defined", i);
            } else if (*reix & D_ARBI) {
                char **fienm = mular_getix(st->str, *reix & ~D_ARBI);

                if  (!!fienm && !!*fienm) {
                    for (int k = 0; k < pgoblin_db_resinfo(rin, PGOBLIN_Nfields); ++k) {
                        const char *reanm = pgoblin_db_subinfo(rin, PGOBLIN_ColName, k);

                        if  (!!reanm && !strcmp(*fienm, reanm)) {
                            res[i] = k;
                            goto break2;
                }   }   }
                ifBLIN_QX1("Field %u=%s~ absent", i, *fienm);
            } else {
                res[i] = (int)*reix;
            }
break2:;
    }   }
out:
    return(res);
#   undef blin_internal_flags
}

static char *
/*****************************************************************************************************
 **                                                                                                 **/
dsig(u_int32_t fl, mular_descriptor *s, int32_t f, int32_t b, pgo0_ent *pr, int line) {                      /**
 **                                                                                                 **
 *****************************************************************************************************
 **       .    f       **
 **    b  ,       +f  canf   **
 ** (     )  -b  canb (       **
 ** ).    ,      .   ,  **
 **  ,      FORWARD  BAKWARD.           **
 **   ,   ,    .                 **
 **   ,   + .     ,      **
 **     .                                                              **
 *****************************************************************************************************/
    int32_t   canf = 0;
    int32_t   canb = 0;
    int32_t   can = 0;
    char    **ss = NULL;
   /******************************************************/
    u_int     tmpidx;  /* Due to the error in cc -v      *
    *** Using built-in specs.                          ***
    *** Configured with: FreeBSD/amd64 system compiler ***
    *** Thread model: posix                            ***
    *** gcc version 3.4.6 [FreeBSD] 20060305           ***
    * I can't use direct                                 *
    *** mular_getix(s, ~can)                           ***
    ******************************************************/

#   define blin_internal_flags (fl & BLIN_MASK)
    if  ((fl & FIRST) && (f < (1 << PGOBLIN_NTBSIZE))) {
        canf = (int32_t)pr->spec[f].f;
    }
    if  ((fl & LAST) && (b < (1 << PGOBLIN_NTBSIZE))) {
        canb = (int32_t)pr->spec[b].b;
    }
    if  (!canf) can = canb;
    else if (!canb) can = canf;
    else if ((fl & FORWARD) && !(fl & BAKWARD)) can = canf;
    else if (!(fl & FORWARD) && (fl & BAKWARD)) can = canb;
    else if (canb > canf) can = canb;
    else can = canf;
    if  (!can) can = (int32_t)pr->def;
    tmpidx = (u_int)~can;
    if  (can) ss = (char**)mular_getix(s, tmpidx);
    if  (!ss) {
        ifBLIN_QX1("tbl NULL @ f=%d b=%d line %d", f, b, line);
        return(NULL);
    } else {
        return(*ss);
    }
#   undef blin_internal_flags
}

static ssize_t
/**************************************************************************
 **                                                                      **/
table(pgoblin_exenv *exenv, pgoblin_nr *rn, ssize_t low, ssize_t hig) { /**
 **                                                                      **
 **************************************************************************/
#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
    ssize_t          hirowborder;       /* ,             */
    ssize_t          textlength = 0;    /*                                   */
    ssize_t          fullrownum = 0;    /*                                 */
    char            *fillement;         /*  ,   possibleU          */
    char            *possibleU;         /*  U ,                     */
    ssize_t          tbwidth;           /*                                              */
    int64_t          resinfo;
    int             *renum      = NULL; /*                          */
    u_int32_t        flow       = 0;
    u_int32_t        fwiv;
    pgoblin_rio     *rou;
    pgoblin_rio     *rin;
    pgoblin_rst     *rst;
    ssize_t          ex         = -EX_DATAERR;
    pgoblin_realsty *st;
    u_int32_t        f;

#   define TRC(E, F, B) { /*   */                                                    \
        ifBLIN_QO3 {                                                                                  \
            int SOUT_i;                                                                               \
            ifBLIN_QX3("<");                                                                          \
            for (SOUT_i = 0; SOUT_i < ENTITY_LETTERS; SOUT_i++) {                                     \
                if  ((E) == entity[SOUT_i]) ifBLIN_QX3("%c", SOUT_i + '@');                           \
            }                                                                                         \
            ifBLIN_QX3( "%c,%d%c,%d%c>"                                                               \
                      , st->a[E].def ? '+' : '-'                                                      \
                      , (int)(F), st->a[E].spec[F].f ? '+' : '-'                                      \
                      , (int)(B), st->a[E].spec[B].b ? '+' : '-'                                      \
                      );                                                                              \
    }   }

#   define SOUT(C, L) do { /*          */               \
        ex = sout(rou, (C), (L));                                                                     \
        if  (ex < 0) goto out;                                                                        \
        textlength += ex;                                                                             \
    } while(0)

#   define VOUT(C, V) do {                                                                            \
        if  (C) {                                                                                     \
            flow |= HALWAYS;                                                                          \
            SOUT((V), 0);                                                                             \
            (V) = NULL;                                                                               \
            SOUT((C), 0);                                                                             \
        }                                                                                             \
    } while(0)

    GET_STY(rst, exenv->options, rn[PGO_CSTY]);
    GET_RIO(rou, exenv->options, rn[PGO_COUT]);
    GET_RIO(rin, exenv->options, rn[PGO_CIN]);
    ex = 0;
    st = rst->style;
    if  (!!strncmp(st->id, PGOBLIN_STYLE_ID, strlen(PGOBLIN_STYLE_ID))) {
        ifBLIN_QX0("Illegal style %s", st->id);
        ERROUT(EX_DATAERR, EINVAL);
    }
    if  ((st->flags & PGOBLIN_NTBSZMK) != PGOBLIN_NTBSIZE) {
        ifBLIN_QX0("Illegal style size %d", st->flags & PGOBLIN_NTBSZMK);
        ERROUT(EX_DATAERR, EINVAL);
    }
    renum = makerenum(exenv, rst, rn[PGO_CIN]);
    if  (low >= 0) {
        flow |= FIRST | FORWARD;
        fullrownum = low;
    }
    if  (0 > (resinfo = pgoblin_db_resinfo(rin, PGOBLIN_Ntuples))) {
        ifBLIN_QW0("resinfo Ntuples");
        ex = EX_SOFTWARE;
        goto out;
    }
    hirowborder = (ssize_t)(fullrownum + resinfo);
    if  ((hig == -2) || ((hig >= 0) && (hirowborder == hig))) {
        flow |= LAST | BAKWARD;
    }
    if  (!!renum) {
        tbwidth = (ssize_t)MULAR_NEXT(st->rid);
    } else if (0 > (resinfo = pgoblin_db_resinfo(rin, PGOBLIN_Nfields))) {
        ifBLIN_QW0("resinfo Nfields");
        ex = EX_SOFTWARE;
        goto out;
    } else {
        tbwidth = (ssize_t)resinfo;
    }
    f = st->flags;
    flow &= (f >> HSHIFT) | P_A;
    fwiv = (f >> VSHIFT) | P_A;
    ifBLIN_QX2("pgo0 %08X %08X %"BLIN_D"..%"BLIN_D"x%"BLIN_D
              , f
              , flow
              , fullrownum
              , hirowborder
              , tbwidth
              );
    if  (hirowborder == 0) {
        flow &= P_A;
        if  (f & HFIRST) {
            fillement = dsig(flow | FORWARD, st->str, 0, 0, &st->a[H], __LINE__);
            TRC(H, 0, 0);
            SOUT(fillement, 0);
        }
        if  (f & HLAST) {
            fillement = dsig(flow | BAKWARD, st->str, 0, 0, &st->a[H], __LINE__);
            TRC(H, 0, 0);
            SOUT(fillement, 0);
        }
    } else if (tbwidth <= 0) {
        flow &= P_A;
        if  (f & HFIRST) {
            fillement = dsig(flow, st->str, 0, (int32_t)hirowborder, &st->a[H], __LINE__);
            TRC(H, 0, hirowborder);
            SOUT(fillement, 0);
        }
        for (; fullrownum < hirowborder; fullrownum++) {
            fillement = dsig( flow
                            , st->str
                            , (int32_t)fullrownum
                            , (int32_t)(hirowborder - fullrownum)
                            , &st->a[H]
                            , __LINE__
                            );
            TRC(H, fullrownum, hirowborder - fullrownum);
            SOUT(fillement, 0);
            possibleU = dsig( flow
                            , st->str
                            , (int32_t)fullrownum
                            , (int32_t)(hirowborder - fullrownum - 1)
                            , &st->a[U]
                            , __LINE__
                            );
            TRC(U, fullrownum, hirowborder - fullrownum - 1);
            flow &= ~HALWAYS;
            if  (f & VFIRST) {
                fillement = dsig(P_A, st->str, 0, (int32_t)tbwidth, &st->a[V], __LINE__);
                TRC(V, 0, tbwidth);
                VOUT(fillement, possibleU);
            }
            if  (f & VLAST) {
                fillement = dsig(P_A, st->str, (int32_t)tbwidth, 0, &st->a[V], __LINE__);
                TRC(V, tbwidth, 0);
                VOUT(fillement, possibleU);
            }
            fillement = dsig( flow
                            , st->str
                            , (int32_t)fullrownum
                            , (int32_t)(hirowborder - fullrownum - 1)
                            , &st->a[D]
                            , __LINE__
                            );
            TRC(D, fullrownum, hirowborder - fullrownum - 1);
            if  (flow & HALWAYS) SOUT(fillement, 0);
        }
        if  (f & HLAST) {
            fillement = dsig(flow, st->str, (int32_t)hirowborder, 0, &st->a[H], __LINE__);
            TRC(H, hirowborder, 0);
            SOUT(fillement, 0);
        }
    } else {
        for (ssize_t lorow = fullrownum; fullrownum < hirowborder; fullrownum++) {
            ssize_t backrownum = hirowborder - fullrownum;

            fillement = dsig( flow
                            , st->str
                            , (int32_t)fullrownum
                            , (int32_t)backrownum
                            , &st->a[H]
                            , __LINE__
                            );
            TRC(H, fullrownum, backrownum);
            SOUT(fillement, 0);
            possibleU = dsig( flow
                            , st->str
                            , (int32_t)fullrownum
                            , (int32_t)backrownum - 1
                            , &st->a[U]
                            , __LINE__
                            );
            TRC(U, fullrownum, backrownum - 1);
            flow = (flow & ~HALWAYS) | (st->flags & HALWAYS);
            for (int colnum = 0; colnum < tbwidth; colnum++) {
                ssize_t localrownum = fullrownum - lorow;
                ssize_t backcolnum = tbwidth - colnum;

                if  (ex < 0) goto out;
                fillement = dsig(fwiv, st->str, colnum, (int32_t)backcolnum, &st->a[V], __LINE__);
                TRC(V, colnum, backcolnum);
                VOUT(fillement, possibleU);
                if  ( !renum
                    ? !pgoblin_db_valinfo(rin, PGOBLIN_IsNull, localrownum, colnum)
                    : !(  (renum[colnum] < 0)
                       || !!pgoblin_db_valinfo(rin, PGOBLIN_IsNull, localrownum, renum[colnum])
                    )  ) {
                    flow |= HALWAYS;
                    SOUT(possibleU, 0);
                    possibleU = NULL;
                    fillement = dsig( fwiv
                                    , st->str
                                    , colnum
                                    , (int32_t)backcolnum - 1
                                    , &st->a[L]
                                    , __LINE__
                                    );
                    TRC(L, colnum, backcolnum - 1);
                    SOUT(fillement, 0);
                    if  (!renum) {
                        SOUT( pgoblin_db_getvalue(rin, localrownum, colnum)
                            , pgoblin_db_valinfo(rin, PGOBLIN_Length, localrownum, colnum)
                            );
                    } else {
                        SOUT( pgoblin_db_getvalue(rin, localrownum, renum[colnum])
                            , pgoblin_db_valinfo(rin, PGOBLIN_Length, localrownum, renum[colnum]
                            )                   );
                    }
                    fillement = dsig( fwiv
                                    , st->str
                                    , colnum
                                    , (int32_t)backcolnum - 1
                                    , &st->a[R]
                                    , __LINE__
                                    );
                    TRC(R, colnum, backcolnum - 1);
                    SOUT(fillement, 0);
                } else if (!!(fillement = dsig( fwiv
                                              , st->str
                                              , colnum
                                              , (int32_t)backcolnum - 1
                                              , &st->a[N]
                                              , __LINE__
                          )  )                ) {
                    TRC(N, colnum, backcolnum);
                    VOUT(fillement, possibleU);
                } else if (f & VALWAYS) {
                    fillement = dsig( fwiv
                                    , st->str
                                    , colnum
                                    , (int32_t)backcolnum - 1
                                    , &st->a[L]
                                    , __LINE__
                                    );
                    TRC(L, colnum, backcolnum);
                    VOUT(fillement, possibleU);
                    fillement = dsig(fwiv
                                    , st->str
                                    , colnum
                                    , (int32_t)backcolnum - 1
                                    , &st->a[R]
                                    , __LINE__
                                    );
                    TRC(R, colnum, backcolnum - 1);
                    VOUT(fillement, possibleU);
            }   }
            fillement = dsig(fwiv, st->str, (int32_t)tbwidth, 0, &st->a[V], __LINE__);
            TRC(V, tbwidth, 0);
            VOUT(fillement, possibleU);
            fillement = dsig( flow
                            , st->str
                            , (int32_t)fullrownum
                            , (int32_t)backrownum - 1
                            , &st->a[D]
                            , __LINE__
                            );
            TRC(D, fullrownum, backrownum - 1);
            if  (flow & HALWAYS) SOUT(fillement, 0);
        }
        if  (flow & LAST) {
            fillement = dsig(flow, st->str, (int32_t)hirowborder, 0, &st->a[H], __LINE__);
            TRC(H, hirowborder, 0);
            SOUT(fillement, 0);
    }   }
out:
    free(renum);
    ifBLIN_QX3("- %"BLIN_D" %"BLIN_D, ex, textlength);
    return((ex < 0) ? ex : textlength);
#   undef blin_internal_flags
}

static int
/**********************************************************************
 **                                                                  **/
init(pgoblin_exenv *exenv, u_int slot) {                            /**
 **                                                                  **
 **********************************************************************/
    (void)exenv;
    (void)slot;
    return(0);
}

static int
/**********************************************************************
 **                                                                  **/
fini(pgoblin_exenv *exenv) {                                        /**
 **                                                                  **
 **********************************************************************/
    (void)exenv;
    return(0);
}

pgoblin_styles PGOBLIN_MODULE =
{ 0                /* flags  */
, "5pGoblin-" VERS /* id     */
, "pgo0\0" VERS    /* name   */
, 0                /* dummy  */
, init
, fini
, parser
, dump
, clear
, get
, table
, NULL             /* load   */
, NULL             /* store  */
, NULL             /* dummy0 */
, NULL             /* dummy1 */
};
